==> openturns_0.12.2.orig.tar.gz: openturns-0.12.2/lib/src/Uncertainty/Distribution/FrankCopula.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  FrankCopula.cxx
 *  @brief The FrankCopula distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-10-31 11:52:04 +0100 (ven 31 oct 2008) $
 *  Id:      $Id: FrankCopula.cxx 995 2008-10-31 10:52:04Z dutka $
 */
#include <cmath>
#include "FrankCopula.hxx"
#include "RandomGenerator.hxx"
#include "PersistentObjectFactory.hxx"
#include "Exception.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      typedef Base::Stat::RandomGenerator RandomGenerator;

      CLASSNAMEINIT(FrankCopula);

      static Base::Common::Factory<FrankCopula> RegisteredFactory("FrankCopula");

      /* Default constructor */
      FrankCopula::FrankCopula()
	: ArchimedeanCopula("FrankCopula"),
	  theta_(2.0)
      {
	setDimension( 2 );
      }

      /* Parameters constructor */
      FrankCopula::FrankCopula(const NumericalScalar theta)
	throw(InvalidArgumentException)
	: ArchimedeanCopula("FrankCopula"),
	  theta_(theta)
      {
	// We set the dimension of the FrankCopula distribution
	setDimension( 2 );
      }

      /* Comparison operator */
      Bool FrankCopula::operator ==(const FrankCopula & other) const
      {
	Bool sameObject = false;

	if (this != &other) { // Other is NOT me, so I have to realize the comparison
	  if ( (theta_ == other.theta_) )
	    sameObject = true;

	} else sameObject = true;

	return sameObject;
      }
  
      /* String converter */
      String FrankCopula::str() const
      {
	OSS oss;
	oss << "class=" << FrankCopula::GetClassName()
	    << " name=" << getName()
	    << " dimension=" << getDimension()
	    << " theta=" << theta_;
	return oss;
      }
  
      /* Virtual constructor */
      FrankCopula * FrankCopula::clone() const
      {
	return new FrankCopula(*this);
      }

      /* Get one realization of the distribution */
      FrankCopula::NumericalPoint FrankCopula::getRealization() const
      {
	NumericalPoint realization(2);
	// We use the general algorithm based on conditional CDF inversion
	const NumericalScalar u(RandomGenerator::Generate());
	realization[0] = u;
	const NumericalScalar v(RandomGenerator::Generate());
	const NumericalScalar factor((v - 1.0) * exp(-theta_ * u));
        realization[1] = 1.0 + log((factor - v) / (factor * exp(theta_) - v)) / theta_;
	return realization;
      }

      /* Get the DDF of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeDDF(const NumericalPoint & point) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// A copula has a null DDF outside of ]0, 1[^2
	if ((u <= 0.0) || (u >= 1.0) || (v <= 0.0) || (v >= 1.0))
	  {
	    return NumericalPoint(2, 0.0);
	  }
	// Optimized version given by Maple 11, as there are a lot of exp's involved
	const NumericalScalar theta2(theta_ * theta_);
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * u));
	const NumericalScalar expMinusThetaV(exp(-theta_ * v));
	const NumericalScalar product1(expMinusThetaU * expMinusThetaV);
	const NumericalScalar sum1(expMinusTheta + product1 - expMinusThetaU - expMinusThetaV);
	const NumericalScalar product2(sum1 * sum1);
	const NumericalScalar factor1(-theta2 * expMinusThetaU * expMinusThetaV * (expMinusTheta - 1.0) / (sum1 * product2));
	NumericalPoint result(2);
	result[0] = factor1 * (-expMinusTheta + product1 - expMinusThetaU + expMinusThetaV);
	result[1] = factor1 * (-expMinusTheta + product1 + expMinusThetaU - expMinusThetaV);
	return result;
      }

      /* Get the PDF of the distribution */
      NumericalScalar FrankCopula::computePDF(const NumericalPoint & point) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// A copula has a null PDF outside of ]0, 1[^2
	if ((u <= 0.0) || (u >= 1.0) || (v <= 0.0) || (v >= 1.0))
	  {
	    return 0.0;
	  }
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * point[0]));
	const NumericalScalar expMinusThetaV(exp(-theta_ * point[1]));
	const NumericalScalar sum1(expMinusTheta + expMinusThetaU * expMinusThetaV - expMinusThetaU - expMinusThetaV);
	return theta_ * expMinusThetaU * expMinusThetaV * (1.0 - expMinusTheta) / (sum1 * sum1);
      }

      /* Get the CDF of the distribution */
      NumericalScalar FrankCopula::computeCDF(const NumericalPoint & point, const Bool tail) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// If we are outside of the support, in the lower parts
	if ((u <= 0.0) || (v <= 0.0))
	  {
	    return 0.0;
	  }
	// If we are outside of the support, in the upper part
	if ((u >= 1.0) && (v >= 1.0))
	  {
	    return 1.0;
	  }
	// If we are outside of the support for u, in the upper part
	if (u >= 1.0)
	  {
	    return v;
	  }
	// If we are outside of the support for v, in the upper part
	if (v >= 1.0)
	  {
	    return u;
	  }
	// If we are in the support
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * point[0]));
	const NumericalScalar expMinusThetaV(exp(-theta_ * point[1]));
	return -log(1.0 + (expMinusThetaU - 1.0) * (expMinusThetaV - 1.0) / (expMinusTheta - 1.0)) / theta_;
      }

      /** Get the PDFGradient of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computePDFGradient(const NumericalPoint & point) const
      {
	throw Base::Common::NotYetImplementedException(HERE);
      }

      /** Get the CDFGradient of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeCDFGradient(const NumericalPoint & point) const
      {
	throw Base::Common::NotYetImplementedException(HERE);
      }

      /* Get the quantile of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeQuantile(const NumericalScalar prob) const
      {
	const NumericalScalar expThetaProb(exp(theta_ * prob));
	const NumericalScalar expTheta(exp(theta_));
	const NumericalScalar sqrtRatio(sqrt((expThetaProb - 1.0) * expTheta / ((expTheta - 1.0) * expThetaProb)));
	return NumericalPoint(2, 1.0 - log(sqrtRatio * (1.0 - expTheta) + expTheta) / theta_);
      }

      /* Compute the CDF of Xi | X1, ..., Xi-1. x = Xi, y = (X1,...,Xi-1) */
      NumericalScalar FrankCopula::computeConditionalCDF(const NumericalScalar x, const NumericalPoint & y) const
      {
	const UnsignedLong conditioningDimension(y.getDimension());
	if (conditioningDimension >= getDimension()) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional CDF with a conditioning point of dimension greater or equal to the distribution dimension.";
	// Special case for no conditioning or independent copula
	if ((conditioningDimension == 0) || (hasIndependentCopula())) return x;
	const NumericalScalar u(y[0]);
	const NumericalScalar v(x);
	// If we are in the support
	const NumericalScalar alpha(exp(-theta_ * v));
	const NumericalScalar beta(exp(-theta_ * u) * (alpha - 1.0));
	return -beta / (alpha - exp(-theta_) - beta);
      }

      /* Compute the quantile of Xi | X1, ..., Xi-1, i.e. x such that CDF(x|y) = q with x = Xi, y = (X1,...,Xi-1) */
      NumericalScalar FrankCopula::computeConditionalQuantile(const NumericalScalar q, const NumericalPoint & y) const
      {
	const UnsignedLong conditioningDimension(y.getDimension());
	if (conditioningDimension >= getDimension()) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile with a conditioning point of dimension greater or equal to the distribution dimension.";
	if ((q < 0.0) || (q > 1.0)) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile for a probability level outside of [0, 1]";
	if (q == 0.0) return 0.0;
	if (q == 1.0) return 1.0;
	// Initialize the conditional quantile with the quantile of the i-th marginal distribution
	// Special case when no contitioning or independent copula
	if ((conditioningDimension == 0) || hasIndependentCopula()) return q;
	const NumericalScalar u(y[0]);
	const NumericalScalar factor((q - 1.0) * exp(-theta_ * u));
	return 1.0 + log((factor - q) / (factor * exp(theta_) - q)) / theta_;
      }

      /* Compute the archimedean generator of the archimedean copula, i.e.
       * the function phi such that the CDF of the copula can
       * be written as CDF(t) = phi^{-1}(phi(u)+phi(v))
       */
      NumericalScalar FrankCopula::computeArchimedeanGenerator(const NumericalScalar t) const
      {
	return log((exp(-theta_) -1.0) / (exp(-theta_ * t) - 1.0));
      }

      /* Compute the inverse of the archimedean generator */
      NumericalScalar FrankCopula::computeInverseArchimedeanGenerator(const NumericalScalar t) const
      {
	return 1.0 + (t - log(1.0 - exp(theta_) * (1 - exp(t)))) / theta_;
      }

      /* Compute the derivative of the density generator */
      NumericalScalar FrankCopula::computeArchimedeanGeneratorDerivative(const NumericalScalar t) const
      {
        return theta_ / (1.0 - exp(-theta_ * t));
      }

      /* Compute the seconde derivative of the density generator */
      NumericalScalar FrankCopula::computeArchimedeanGeneratorSecondDerivative(const NumericalScalar t) const
      {
	NumericalScalar expThetaT(exp(theta_ * t));
	NumericalScalar ratio(theta_ / (expThetaT - 1.0));
	return ratio * ratio * expThetaT;
      }

      /** Parameters value and description accessor */
      FrankCopula::NumericalPointWithDescriptionCollection FrankCopula::getParametersCollection() const
      {
	NumericalPointWithDescriptionCollection parameters(1);
	NumericalPointWithDescription point(1);
        Description description(point.getDimension());
	point[0] = theta_;
	description[0] = "theta";
	point.setDescription(description);
	point.setName(getDescription()[0]);
	parameters[0] = point;
	return parameters;
      }

      /* Tell if the distribution has independent copula */
      Bool FrankCopula::hasIndependentCopula() const
      {
	return (theta_ == 0.0);
      }

      /* Theta accessor */
      void FrankCopula::setTheta(const NumericalScalar theta)
	throw(InvalidArgumentException)
      {
	if (theta <= 0.) throw InvalidArgumentException(HERE) << "Theta MUST be positive";
	theta_ = theta;
      }

      /* Theta accessor */
      NumericalScalar FrankCopula::getTheta() const
      {
	return theta_;
      }

      /* Method save() stores the object through the StorageManager */
      void FrankCopula::save(const StorageManager::Advocate & adv) const
      {
	ArchimedeanCopula::save(adv);
	adv.writeValue("theta_", theta_);
      }

      /* Method load() reloads the object from the StorageManager */
      void FrankCopula::load(const StorageManager::Advocate & adv)
      {
	ArchimedeanCopula::load(adv);

	String name;
	NumericalScalar value;
	StorageManager::List objList = adv.getList(StorageManager::NumericalScalarEntity);
	for(objList.firstValueToRead(); objList.moreValuesToRead(); objList.nextValueToRead()) {
	  if (objList.readValue(name, value)) {
	    if (name == "theta_") theta_ = value;
	  }
	}
      }
      



    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

==> openturns_0.13.1.orig.tar.gz: openturns-0.13.1/lib/src/Uncertainty/Distribution/FrankCopula.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  FrankCopula.cxx
 *  @brief The FrankCopula distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 14:47:53 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: FrankCopula.cxx 1262 2009-05-28 12:47:53Z dutka $
 */
#include <cmath>
#include "FrankCopula.hxx"
#include "RandomGenerator.hxx"
#include "PersistentObjectFactory.hxx"
#include "Exception.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      typedef Base::Stat::RandomGenerator RandomGenerator;

      CLASSNAMEINIT(FrankCopula);

      static Base::Common::Factory<FrankCopula> RegisteredFactory("FrankCopula");

      /* Default constructor */
      FrankCopula::FrankCopula()
	: ArchimedeanCopula("FrankCopula"),
	  theta_(2.0)
      {
	setDimension( 2 );
      }

      /* Parameters constructor */
      FrankCopula::FrankCopula(const NumericalScalar theta)
	throw(InvalidArgumentException)
	: ArchimedeanCopula("FrankCopula"),
	  theta_(theta)
      {
	// We set the dimension of the FrankCopula distribution
	setDimension( 2 );
      }

      /* Comparison operator */
      Bool FrankCopula::operator ==(const FrankCopula & other) const
      {
	Bool sameObject = false;

	if (this != &other) { // Other is NOT me, so I have to realize the comparison
	  if ( (theta_ == other.theta_) )
	    sameObject = true;

	} else sameObject = true;

	return sameObject;
      }
  
      /* String converter */
      String FrankCopula::__repr__() const
      {
	OSS oss;
	oss << "class=" << FrankCopula::GetClassName()
	    << " name=" << getName()
	    << " dimension=" << getDimension()
	    << " theta=" << theta_;
	return oss;
      }
  
      /* Virtual constructor */
      FrankCopula * FrankCopula::clone() const
      {
	return new FrankCopula(*this);
      }

      /* Get one realization of the distribution */
      FrankCopula::NumericalPoint FrankCopula::getRealization() const
      {
	NumericalPoint realization(2);
	// We use the general algorithm based on conditional CDF inversion
	const NumericalScalar u(RandomGenerator::Generate());
	realization[0] = u;
	const NumericalScalar v(RandomGenerator::Generate());
	const NumericalScalar factor((v - 1.0) * exp(-theta_ * u));
        realization[1] = 1.0 + log((factor - v) / (factor * exp(theta_) - v)) / theta_;
	return realization;
      }

      /* Get the DDF of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeDDF(const NumericalPoint & point) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// A copula has a null DDF outside of ]0, 1[^2
	if ((u <= 0.0) || (u >= 1.0) || (v <= 0.0) || (v >= 1.0))
	  {
	    return NumericalPoint(2, 0.0);
	  }
	// Optimized version given by Maple 11, as there are a lot of exp's involved
	const NumericalScalar theta2(theta_ * theta_);
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * u));
	const NumericalScalar expMinusThetaV(exp(-theta_ * v));
	const NumericalScalar product1(expMinusThetaU * expMinusThetaV);
	const NumericalScalar sum1(expMinusTheta + product1 - expMinusThetaU - expMinusThetaV);
	const NumericalScalar product2(sum1 * sum1);
	const NumericalScalar factor1(-theta2 * expMinusThetaU * expMinusThetaV * (expMinusTheta - 1.0) / (sum1 * product2));
	NumericalPoint result(2);
	result[0] = factor1 * (-expMinusTheta + product1 - expMinusThetaU + expMinusThetaV);
	result[1] = factor1 * (-expMinusTheta + product1 + expMinusThetaU - expMinusThetaV);
	return result;
      }

      /* Get the PDF of the distribution */
      NumericalScalar FrankCopula::computePDF(const NumericalPoint & point) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// A copula has a null PDF outside of ]0, 1[^2
	if ((u <= 0.0) || (u >= 1.0) || (v <= 0.0) || (v >= 1.0))
	  {
	    return 0.0;
	  }
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * point[0]));
	const NumericalScalar expMinusThetaV(exp(-theta_ * point[1]));
	const NumericalScalar sum1(expMinusTheta + expMinusThetaU * expMinusThetaV - expMinusThetaU - expMinusThetaV);
	return theta_ * expMinusThetaU * expMinusThetaV * (1.0 - expMinusTheta) / (sum1 * sum1);
      }

      /* Get the CDF of the distribution */
      NumericalScalar FrankCopula::computeCDF(const NumericalPoint & point, const Bool tail) const
      {
	const NumericalScalar u(point[0]);
	const NumericalScalar v(point[1]);
	// If we are outside of the support, in the lower parts
	if ((u <= 0.0) || (v <= 0.0))
	  {
	    return 0.0;
	  }
	// If we are outside of the support, in the upper part
	if ((u >= 1.0) && (v >= 1.0))
	  {
	    return 1.0;
	  }
	// If we are outside of the support for u, in the upper part
	if (u >= 1.0)
	  {
	    return v;
	  }
	// If we are outside of the support for v, in the upper part
	if (v >= 1.0)
	  {
	    return u;
	  }
	// If we are in the support
	const NumericalScalar expMinusTheta(exp(-theta_));
	const NumericalScalar expMinusThetaU(exp(-theta_ * point[0]));
	const NumericalScalar expMinusThetaV(exp(-theta_ * point[1]));
	return -log(1.0 + (expMinusThetaU - 1.0) * (expMinusThetaV - 1.0) / (expMinusTheta - 1.0)) / theta_;
      }

      /** Get the PDFGradient of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computePDFGradient(const NumericalPoint & point) const
      {
	throw Base::Common::NotYetImplementedException(HERE);
      }

      /** Get the CDFGradient of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeCDFGradient(const NumericalPoint & point) const
      {
	throw Base::Common::NotYetImplementedException(HERE);
      }

      /* Get the quantile of the distribution */
      FrankCopula::NumericalPoint FrankCopula::computeQuantile(const NumericalScalar prob,
							       const Bool tail) const
      {
	const NumericalScalar expThetaProb(exp(theta_ * prob));
	const NumericalScalar expTheta(exp(theta_));
	const NumericalScalar sqrtRatio(sqrt((expThetaProb - 1.0) * expTheta / ((expTheta - 1.0) * expThetaProb)));
	return NumericalPoint(2, 1.0 - log(sqrtRatio * (1.0 - expTheta) + expTheta) / theta_);
      }

      /* Compute the CDF of Xi | X1, ..., Xi-1. x = Xi, y = (X1,...,Xi-1) */
      NumericalScalar FrankCopula::computeConditionalCDF(const NumericalScalar x, const NumericalPoint & y) const
      {
	const UnsignedLong conditioningDimension(y.getDimension());
	if (conditioningDimension >= getDimension()) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional CDF with a conditioning point of dimension greater or equal to the distribution dimension.";
	// Special case for no conditioning or independent copula
	if ((conditioningDimension == 0) || (hasIndependentCopula())) return x;
	const NumericalScalar u(y[0]);
	const NumericalScalar v(x);
	// If we are in the support
	const NumericalScalar alpha(exp(-theta_ * v));
	const NumericalScalar beta(exp(-theta_ * u) * (alpha - 1.0));
	return -beta / (alpha - exp(-theta_) - beta);
      }

      /* Compute the quantile of Xi | X1, ..., Xi-1, i.e. x such that CDF(x|y) = q with x = Xi, y = (X1,...,Xi-1) */
      NumericalScalar FrankCopula::computeConditionalQuantile(const NumericalScalar q, const NumericalPoint & y) const
      {
	const UnsignedLong conditioningDimension(y.getDimension());
	if (conditioningDimension >= getDimension()) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile with a conditioning point of dimension greater or equal to the distribution dimension.";
	if ((q < 0.0) || (q > 1.0)) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile for a probability level outside of [0, 1]";
	if (q == 0.0) return 0.0;
	if (q == 1.0) return 1.0;
	// Initialize the conditional quantile with the quantile of the i-th marginal distribution
	// Special case when no contitioning or independent copula
	if ((conditioningDimension == 0) || hasIndependentCopula()) return q;
	const NumericalScalar u(y[0]);
	const NumericalScalar factor((q - 1.0) * exp(-theta_ * u));
	return 1.0 + log((factor - q) / (factor * exp(theta_) - q)) / theta_;
      }

      /* Compute the archimedean generator of the archimedean copula, i.e.
       * the function phi such that the CDF of the copula can
       * be written as CDF(t) = phi^{-1}(phi(u)+phi(v))
       */
      NumericalScalar FrankCopula::computeArchimedeanGenerator(const NumericalScalar t) const
      {
	return log((exp(-theta_) -1.0) / (exp(-theta_ * t) - 1.0));
      }

      /* Compute the inverse of the archimedean generator */
      NumericalScalar FrankCopula::computeInverseArchimedeanGenerator(const NumericalScalar t) const
      {
	return 1.0 + (t - log(1.0 - exp(theta_) * (1 - exp(t)))) / theta_;
      }

      /* Compute the derivative of the density generator */
      NumericalScalar FrankCopula::computeArchimedeanGeneratorDerivative(const NumericalScalar t) const
      {
        return theta_ / (1.0 - exp(-theta_ * t));
      }

      /* Compute the seconde derivative of the density generator */
      NumericalScalar FrankCopula::computeArchimedeanGeneratorSecondDerivative(const NumericalScalar t) const
      {
	NumericalScalar expThetaT(exp(theta_ * t));
	NumericalScalar ratio(theta_ / (expThetaT - 1.0));
	return ratio * ratio * expThetaT;
      }

      /** Parameters value and description accessor */
      FrankCopula::NumericalPointWithDescriptionCollection FrankCopula::getParametersCollection() const
      {
	NumericalPointWithDescriptionCollection parameters(1);
	NumericalPointWithDescription point(1);
        Description description(point.getDimension());
	point[0] = theta_;
	description[0] = "theta";
	point.setDescription(description);
	point.setName(getDescription()[0]);
	parameters[0] = point;
	return parameters;
      }

      void FrankCopula::setParametersCollection(const NumericalPointCollection & parametersCollection)
      {
	*this = FrankCopula(parametersCollection[0][0]);
      }

      /* Tell if the distribution has independent copula */
      Bool FrankCopula::hasIndependentCopula() const
      {
	return (theta_ == 0.0);
      }

      /* Theta accessor */
      void FrankCopula::setTheta(const NumericalScalar theta)
	throw(InvalidArgumentException)
      {
	if (theta <= 0.) throw InvalidArgumentException(HERE) << "Theta MUST be positive";
	theta_ = theta;
      }

      /* Theta accessor */
      NumericalScalar FrankCopula::getTheta() const
      {
	return theta_;
      }

      /* Method save() stores the object through the StorageManager */
      void FrankCopula::save(const StorageManager::Advocate & adv) const
      {
	ArchimedeanCopula::save(adv);
	adv.writeValue("theta_", theta_);
      }

      /* Method load() reloads the object from the StorageManager */
      void FrankCopula::load(const StorageManager::Advocate & adv)
      {
	ArchimedeanCopula::load(adv);

	String name;
	NumericalScalar value;
	StorageManager::List objList = adv.getList(StorageManager::NumericalScalarEntity);
	for(objList.firstValueToRead(); objList.moreValuesToRead(); objList.nextValueToRead()) {
	  if (objList.readValue(name, value)) {
	    if (name == "theta_") theta_ = value;
	  }
	}
      }
      



    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */
